# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import os
import tempfile
import zipfile

def _copy(input_fd, output_fd):
  bytes_to_read = 1024
  read_bytes = input_fd.read(bytes_to_read)
  while read_bytes:
    output_fd.write(read_bytes)
    read_bytes = input_fd.read(bytes_to_read)

def _write_partial_file(top_level_dir, zip_fd, fd, archive_filename):
  """The ZipFile class doesn't provide a method to write data to an
  archive directly from a file-like object. As a result, we have to write
  a partial file to disk and then use ZipFile's write() method to write
  the partial file to the archive."""
  
    # Write fd to a tmpfile on disk
  tmp_os_file_descriptor, tmp_file_path = tempfile.mkstemp()
  tmp_file_handle = os.fdopen(tmp_os_file_descriptor, "w")

  try:
    try:
      _copy(fd, tmp_file_handle)
      tmp_file_handle.flush()
          
        # Write file to zip archive
      zip_fd.write(tmp_file_path, top_level_dir + "/" + archive_filename)
    finally:
      tmp_file_handle.close()
                
  finally:
    os.remove(tmp_file_path)
            
def get_compressed_files(path_list, max_bytes_to_collect, top_level_dir):
  """This function returns at most max_bytes_to_collect uncompressed files in
     a compressed zip file. It takes a list of file paths and writes those
     files to the given top level directory in the zip file.

     The archive resides in a temporary directory, and it is the caller's 
     responsibility to delete the archive after using it."""
        
  tmp_os_file_descriptor, tmp_file_path = tempfile.mkstemp()
  tmp_file_handle = None
  zip_fd = None

  try:
    tmp_file_handle = os.fdopen(tmp_os_file_descriptor, "w")
    zip_fd = zipfile.ZipFile(tmp_file_handle, "w", zipfile.ZIP_DEFLATED)

    bytes_collected = 0
    for curr_file_path in path_list:
      if not os.path.exists(curr_file_path):
        continue

      curr_file_length = os.path.getsize(curr_file_path)
      curr_file_name = os.path.basename(curr_file_path)
      if bytes_collected + curr_file_length > max_bytes_to_collect:
        # In this path, we're writing only a partial file because of our byte
        # limit.
        curr_file_handle = open(curr_file_path)
        try:
          remaining_bytes = max_bytes_to_collect - bytes_collected
          curr_file_handle.seek(curr_file_length - remaining_bytes)
          _write_partial_file(top_level_dir, zip_fd, curr_file_handle,
                              curr_file_name)
          bytes_collected += remaining_bytes
          break
        finally:
          curr_file_handle.close()
      else:
        # In this path, we're writing the entire file.
        zip_fd.write(curr_file_path, top_level_dir + "/" + curr_file_name)
        bytes_collected += curr_file_length
  finally:
    if zip_fd:
      zip_fd.close()
    if tmp_file_handle:
      tmp_file_handle.close()

  return tmp_file_path
